@@ -15,9 +15,11 @@ import json
 import datetime
 import base64
 import lzma
-
+import numpy as np
+from PIL import Image, ImageDraw, ImageOps
 from sleekxmppfs import ClientXMPP, Callback, MatchXPath
 from sleekxmppfs.exceptions import XMPPError
+from io import BytesIO
 
 from paho.mqtt.client import Client  as ClientMQTT
 from paho.mqtt import publish as MQTTPublish
@@ -425,6 +427,7 @@ class VacBot():
 
         self.rooms = []
         self.last_clean_image = None
+        self.live_map = None
 
         self.statusEvents = EventEmitter()
         self.batteryEvents = EventEmitter()
@@ -433,16 +436,8 @@ class VacBot():
         self.errorEvents = EventEmitter()
 
         # Map variables
-        self.buffer = []
-        self.height = 0
-        self.width = 0
-        self.row_grid = 0
-        self.column_grid = 0
-        self.row_piece = 0
-        self.column_piece = 0
-        self.width = 0
-
-        self.InitializeMapBuffer(8,8,100,100)
+        self.buffer = np.zeros((1000, 100, 100))
+        self.mapPieces = []
 
         #Set none for clients to start        
         self.iotmq = None
@@ -543,6 +538,11 @@ class VacBot():
 
             self.statusEvents.notify(self.vacuum_status)
 
+    def _handle_minor_map(self, event):
+        response = event['body']['data']
+        
+        self.AddMapPiece(response['pieceIndex'], response['pieceValue'])
+
     def _handle_cached_map(self, event):
         response = event['body']['data']
         mapid = response['info'][0]['mid']
@@ -550,6 +550,10 @@ class VacBot():
         self.rooms = []
         self.exc_command('getMapSet', {'mid': mapid,'type': 'ar'})
 
+        for i in range(64):
+            self.exc_command('getMinorMap', {'mid': mapid,'type': 'ol', 'pieceIndex': i})
+        
+
     def _handle_map_set(self, event):
         response = event['body']['data']
         
@@ -722,63 +726,80 @@ class VacBot():
 
     def AddMapPiece(self,mapPiece, b64):
         decoded = self.decompress7zBase64Data(b64)
+        
+        decoded = list(decoded)
 
-        self.UpdateMapBuffer(mapPiece, decoded)
+        MATRIX_PIECE = np.reshape(decoded,(100,100))
+
+        self.UpdateMapBuffer(mapPiece, MATRIX_PIECE)
 
     def decompress7zBase64Data(self, data):
+        finalArray = bytearray()
+
         # Decode Base64
         data = base64.b64decode(data)
-        
-        # Get lzma output size (as done by the Android app)
-        len_array = data[5:5+4]
-        len_value = ((len_array[0] & 0xFF) | (len_array[1] << 8 & 0xFF00) | (len_array[2] << 16 & 0xFF0000) | (len_array[3] << 24 & 0xFF000000)) 
-        
-        # Init the LZMA decompressor using the lzma header
-        dec = lzma.LZMADecompressor(lzma.FORMAT_RAW, None, [lzma._decode_filter_properties(lzma.FILTER_LZMA1, data[0:5])])
-        
-        # Decompress the lzma stream to get raw data
-        return dec.decompress(data[9:], len_value)
 
-    def InitializeMapBuffer(self, i,i2,i3,i4):
-        self.row_grid = i
-        self.column_grid = i2
-        self.column_piece = i4
-        self.row_piece = i3
+        i = 0
+        for idx in data:
+            if (i == 8):
+                finalArray += b'\x00\x00\x00\x00'
+            finalArray.append(idx)
+            i +=1
 
-        self.pieceCount = i3 * i4
-        self.gridCount = i * i2
+        dec = lzma.LZMADecompressor(lzma.FORMAT_AUTO, None, None)
 
-        self.width = i2 * i4
-        self.height = i * i3
-        
-        self.buffer = [[0 for x in range(self.width)] for y in range(self.height)]
-        _LOGGER.debug('Map Buffer Initialized')
+        return dec.decompress(finalArray)
 
-    def StampMap(self):
-        for i6 in range(800):
-            for i9 in range(800):
-                arr = self.buffer[i6][i9]
-                if arr == 0x01:
-                    print('1', end = '')
-                if arr == 0x02:
-                    print('2', end = '')
-                if arr == 0x00:
-                    print(' ', end = '')
-            print('')
-                
+    def CreatePNGMap(self):
+        im = Image.new("RGB", (6400, 6400))
+        draw = ImageDraw.Draw(im)
 
-    def UpdateMapBuffer(self, i, bArr):
-        rowStart = int(i / self.row_grid)
-        columnStart = int(i % self.column_grid)
-        
-        for row in range(self.row_grid):
-            for column in range(self.column_grid):
-                bufferRow = row + rowStart * self.row_grid
-                bufferColumn = column + columnStart * self.column_grid
-                pieceDataPosition = self.row_grid * row + column
+        imageX = 0
+        imageY = 0
 
-                self.buffer[bufferRow][bufferColumn] = bArr[pieceDataPosition]
-                print(str(bufferRow) + '-' + str(bufferColumn)) 
+        prev = self.mapPieces[0] - 1
+        for mapPiece in self.mapPieces:
+            if mapPiece != self.mapPieces[0]:
+                if mapPiece % 8 != 0:
+                        imageY += 100
+                else:
+                    imageX += 100
+                    imageY = 0
+
+            prev = mapPiece
+
+            _LOGGER.debug('PIECE: ' + str(mapPiece) + ' X:' + str(imageX) + ' Y:' + str(imageY))
+
+            for x in range(100):
+                for y in range(100):
+                    if self.buffer[mapPiece][x][y] == 0x01: #floor
+                        draw.point((imageX+x,imageY+y), fill=(186,218,255))
+                    if self.buffer[mapPiece][x][y] == 0x02: #wall
+                        draw.point((imageX+x,imageY+y), fill=(78,150,226))
+
+        del draw
+
+        #Flip and crop
+        im = ImageOps.flip(im)
+
+        image_data = np.asarray(im)
+        image_data_bw = image_data.max(axis=2)
+        non_empty_columns = np.where(image_data_bw.max(axis=0)>0)[0]
+        non_empty_rows = np.where(image_data_bw.max(axis=1)>0)[0]
+        cropBox = (min(non_empty_rows), max(non_empty_rows), min(non_empty_columns), max(non_empty_columns))
+
+        image_data_new = image_data[cropBox[0]:cropBox[1]+1, cropBox[2]:cropBox[3]+1 , :]
+
+        new_image = Image.fromarray(image_data_new)
+
+        buffered = BytesIO()
+        new_image.save(buffered, format="PNG")
+
+        self.live_map = base64.b64encode(buffered.getvalue())
+
+    def UpdateMapBuffer(self, mapPiece, matrix):
+        self.mapPieces.append(mapPiece)
+        self.buffer[mapPiece] = matrix
 
 #This is used by EcoVacsIOTMQ for _ctl_to_dict
 def RepresentsInt(stringvar):
@@ -1019,6 +1040,8 @@ class EcoVacsIOTMQ(ClientMQTT):
                 jsonstring['event'] = "fan_speed"
             elif 'cachedmapinfo' in eventname:
                 jsonstring['event'] = "cached_map"
+            elif 'minormap' in eventname:
+                jsonstring['event'] = "minor_map"
             elif 'mapset' in eventname:
                 jsonstring['event'] = "map_set"
             elif 'mapsubset' in eventname:
